/*----------------------------------------------------------------------
  File    : sudoku.c
  Contents: solver for Sudoku puzzles
  Author  : Christian Borgelt
  History : 2007.02.28 file created
            2007.09.23 first version completed
            2007.09.24 added dependent field lists and backtracking
            2007.09.25 backtracking debugged, finds all solutions now
            2007.09.26 usage message and solution reporting improved
            2007.09.27 board output improved (square structure)
            2007.09.30 traversal of all backtracking fields added
            2007.10.01 iterative deepening of backtracking added
            2007.10.10 '0' as an alternative to '.' made possible
            2013.08.09 adapted to higher compiler warning level
            2014.10.24 changed from LGPL license to MIT license
----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
#define PRGNAME     "sudoku"
#define DESCRIPTION "simple Sudoku puzzle solver"
#define VERSION     "version 1.6 (2014.10.24)         " \
                    "(c) 2007-2014   Christian Borgelt"

/* --- reasons for setting a field --- */
#define GIVEN       -1          /* digit is given in the puzzle */
#define ROW          8          /* only field in row    for digit */
#define COLUMN      17          /* only field in column for digit */
#define SQUARE      26          /* only field in square for digit */
#define UNIQUE      27          /* digit is the only one left */
#define TRIAL       28          /* digit was (successfully) tried */

/* --- reporting modes --- */
#define FLAGS        1          /* show the digit flags */
#define STEPS        2          /* show the steps to the solution */
#define ALL          4          /* show all possible solutions */
#define ONE          8          /* try only one backtracking field */

/*----------------------------------------------------------------------
  Type Definitions
----------------------------------------------------------------------*/
typedef struct field {          /* --- field of a Sudoku board --- */
  short        digit;           /* contained digit (0: unknown) */
  short        flags;           /* flags for possible digits */
  short        row;             /* row    of the field */
  short        col;             /* column of the field */
  short        step;            /* step in which the field was set */
  short        reason;          /* reason for setting the field */
  short        rsvd1;           /* reserved (dummy for alignment) */
  short        rsvd2;           /* reserved (dummy for alignment) */
  struct field *deps[20];       /* array of dependent fields */
} FIELD;                        /* (field of a Sudoku board) */

/*----------------------------------------------------------------------
  Constants
----------------------------------------------------------------------*/
static const char *digits = ".1234567890";

/*----------------------------------------------------------------------
  Global Variables
----------------------------------------------------------------------*/
static short  unique[512];      /* lookup table for unique digits */
static FIELD  board [81];       /* fields of the Sudoku board */
static FIELD *sects [27][9];    /* board sections (rows/cols/squares) */
static FIELD *steps [81];       /* steps of a solution procedure */
static int    stack [81][81+1]; /* stack for backtracking (oversized) */
static int    unknown = 81;     /* number of unknown fields */
static int    report  =  0;     /* reporting and search mode */
static int    maxdep  = -1;     /* maximum depth of backtracking */

/*----------------------------------------------------------------------
  Functions
----------------------------------------------------------------------*/

static void set (FIELD *field, int digit, int reason)
{                               /* --- set a field to a digit */
  int i, m;                     /* loop variable, bit mask */

  assert((digit  >= 1)     && (digit  <= 9));
  assert((reason >= GIVEN) && (reason <= TRIAL));
  assert(field->digit == 0);    /* the field must not yet be set */
  field->digit  = (short)digit; /* set the field to the given digit */
  m = 1 << (digit-1);           /* compute the corresponding bit */
  field->flags  = (short)(field->flags & m); /* remove other flags */
  field->step   = (short)(81-unknown--);     /* step in field set */
  field->reason = (short)reason;/* reason for setting the field */
  m = ~m;                       /* create the digit removal mask */
  for (i = 20; --i >= 0; )      /* traverse the dependent fields */
    field->deps[i]->flags = (short)(field->deps[i]->flags & m);
}  /* set() */                  /* remove the set digit (mask flags) */

/*--------------------------------------------------------------------*/

static void show (char *title, int mode, int depth)
{                               /* --- show the board */
  int  i, k, n;                 /* loop variables */
  char *s, *c;                  /* buffer for reason text */

  /* --- print the board --- */
  fputs(title, stdout);         /* print the board title */
  if (depth >= 0) printf(" (%dx bt)", depth);
  fputc('\n', stdout);          /* print backtracking depth */
  for (i = n = 0; i < 9; i++) { /* traverse the rows */
    c = ((i == 2) || (i == 5)) ? "_" : " ";
    for (k = 0; k < 9; k++) {   /* traverse the columns */
      if (mode & FLAGS) printf("%03x", board[n].flags);
      printf("%c", digits[board[n++].digit]);
      if (k < 8) fputs(((k == 2) || (k == 5)) ? "|" : c, stdout);
    }                           /* print the field contents */
    fputc('\n', stdout);        /* and a field separator, */
  }                             /* finally terminate the row */

  /* --- print the solution procedure --- */
  if ((mode & STEPS) == 0) return;
  assert(unknown <= 0);         /* all fields must be set */
  fputs("\nprocedure\n",stdout);/* print the procedure title */
  for (i = 0; i < 81; i++)      /* sort fields according to step */
    steps[board[i].step] = board +i;
  for (i = 0; i < 81; i++) {    /* traverse the fields */
    k = steps[i]->reason;       /* only process the fields */
    if (k <= GIVEN) continue;   /* that were not given */
    if      (k <= ROW)    s = "row";
    else if (k <= COLUMN) s = "column";
    else if (k <= SQUARE) s = "square";
    else if (k == UNIQUE) s = "unique";
    else                  s = "trial";
    printf("row %d, ",    steps[i]->row+1);
    printf("column %d: ", steps[i]->col+1);
    printf(" %d   %s\n",  steps[i]->digit, s);
  }                             /* print which field was set and why */
}  /* show() */

/*--------------------------------------------------------------------*/

static void save (int depth)
{                               /* --- save a board state */
  int i, *s;                    /* loop variable, stack element */

  assert((depth >= 0) && (depth < 81));
  s = stack[depth];             /* get the buffer to save to */
  s[i = 81] = unknown;          /* save number of unknown fields */
  while (--i >= 0)              /* save the digit and the flags */
    s[i] = board[i].digit | (board[i].flags << 16);
}  /* save() */

/*--------------------------------------------------------------------*/

static void restore (int depth)
{                               /* --- restore a board state */
  int i, *s;                    /* loop variable, stack element */

  assert((depth >= 0) && (depth < 81));
  s = stack[depth];             /* get the buffer to restore from */
  unknown = s[i = 81];          /* restore number of unknown fields */
  while (--i >= 0) {            /* restore the digit and the flags */
    board[i].digit = (short)(s[i] & 0xf);
    board[i].flags = (short)(s[i] >> 16);
  }
}  /* restore() */

/*--------------------------------------------------------------------*/

static int solve (int depth)
{                               /* --- solve a Sudoku puzzle */
  int   i, k, p;                /* loop variables */
  int   d, t, o;                /* buffers for field contents */
  FIELD *field;                 /* to traverse the fields */

  /* --- direct reasoning --- */
  do {                          /* direct reasoning loop */
    p = unknown;                /* note the number of unknown fields */

    /* -- check for unique digits -- */
    for (i = 81; --i >= 0; ) {  /* traverse the unknown fields */
      if (board[i].digit != 0) continue;
      d = unique[board[i].flags];    /* get the possible digits */
      if (d == 0) return -1;    /* check for at least one digit */
      /* (d == 0) indicates that (board[i].flags == 0) and   */
      /* thus that there is no possible digit for the field. */
      if (d >  0) set(board +i, d, UNIQUE);
    }                           /* if the digit is unique, set it */
    if (unknown <= 0) {         /* check whether puzzle is solved */
      show("\nsolution", report, depth); return 0; }

    /* -- check rows/columns/squares -- */
    for (i = 0; i < 27; i++) {  /* traverse the board sections */
      t = o = 0;                /* initialize the occurrence flags */
      for (k = 9; --k >= 0; ) { /* traverse fields of the section */
        field = sects[i][k];    /* process only the unknown fields */
        if (field->digit != 0) continue;
        t |= field->flags & o;  /* "count" the occurrences */
        o |= field->flags;      /* of the possible digits: */
      }                         /* o: once or more, t: twice or more */
      o &= ~t;                  /* find digits occurring only once */
      if (o == 0) continue;     /* if there are none, skip section */
      for (k = 9; --k >= 0; ) { /* traverse the board section again */
        field = sects[i][k];    /* (known fields have no digit in o) */
        t = field->flags & o;   /* check for a unique digit and */
        d = unique[t];          /* get this digit if it exists */
        if (d <= 0) { if (t != 0) return -1; continue; }
        /* (d <= 0) and (t != 0) indicates that more than one digit  */
        /* can only be in the currently considered field. As this is */
        /* clearly impossible, the function is aborted in this case. */
        set(field, d, i);       /* if a digit is unique, set it */
        o &= ~t;                /* and remove it from the flags */
        if (o == 0) break;      /* if no unique digits are left, */
      }                         /* abort the loop */
    }
    if (unknown <= 0) {         /* check whether puzzle is solved */
      show("\nsolution", report, depth); return 0; }
  } while (p > unknown);        /* while at least one field was set */

  /* --- backtracking --- */
  if (depth >= maxdep)          /* if at maximum backtracking depth, */
    return -2;                  /* abort recursion with failure */
  save(depth);                  /* save the current board state */
  for (i = t = 0; i < 81; i++){ /* traverse the unknown fields */
    if (board[i].digit != 0) continue;
    field = board +i;           /* get the next unknown field */
    d = field->flags; p = -1;   /* get its possible digits */
    for (k = 0; k < 9; k++) {   /* and traverse them */
      if ((d & (1 << k)) == 0) continue;
      if (t) restore(depth);    /* restore the board (if necessary) */
      set(field, t = k+1, TRIAL);
      p &= solve(depth+1);      /* set the next digit tentatively */
      if (p < -1) break;        /* and (try to) solve recursively */
      if ((p == 0) && !(report & ALL)) return 0;
    }                           /* (loop only to find all solutions) */
    if ((p == 0) || (report & ONE)) break;
  }                             /* (loop only if no solution found) */
  return p;                     /* return whether solution was found */
}  /* solve() */

/*--------------------------------------------------------------------*/

static void example ()
{                               /* --- show an example puzzle */
  printf("example Sudoku puzzle (known as \"AI Escargot\")\n"
         "1 . .|. . 7|. 9 .\n"
         ". 3 .|. 2 .|. . 8\n"
         "._._9|6_._.|5_._.\n"
         ". . 5|3 . .|9 . .\n"
         ". 1 .|. 8 .|. . 2\n"
         "6_._.|._._4|._._.\n"
         "3 . .|. . .|. 1 .\n"
         ". 4 .|. . .|. . 7\n"
         ". . 7|. . .|3 . .\n\n");
  printf("Dots (that is, '.') may be replaced by zeros (that is, '0')."
         "\nSpaces and other separators (that is, '|' and '_') as "
         "well as\nline breaks may be omitted or replaced by other "
         "characters.\nThe puzzle description may be followed "
         "by arbitrary comments.\n");
}  /* example() */

/*--------------------------------------------------------------------*/

int main (int argc, char* argv[])
{                               /* --- Sudoku solver main function */
  int   r, c, i, k, n;          /* loop variables */
  int   sr, sc;                 /* square coordinates */
  FIELD *field;                 /* to traverse the fields */
  char  *s;                     /* to traverse the program arguments */
  char  *fname = NULL;          /* name of the input file */
  FILE  *in;                    /* input file */

  /* --- print startup/usage message --- */
  if (argc > 1) {               /* if arguments are given */
    fprintf(stderr, "%s - %s\n", argv[0], DESCRIPTION);
    fprintf(stderr, "%s\n", VERSION); } /* print a startup message */
  else {                        /* if no argument is given */
    printf("usage: %s [options] [-|<file>]\n", argv[0]);
    printf("%s\n", DESCRIPTION);
    printf("%s\n", VERSION);
    printf("-a     report all solutions "
                  "(default: only first solution found)\n");
    printf("-r     report the steps leading to a solution "
                  "(field by field)\n");
    printf("-b#    maximum depth for backtracking "
                  "(default: iterative deepening)\n");
    printf("-o     check only one field when backtracking "
                  "(default: check all)\n");
    printf("-h     print an example puzzle "
                  "(to illustrate the input format)\n");
    printf("file   name of a file containing a Sudoku puzzle\n");
    printf("       (\"-\": read puzzle from standard input)\n");
    return 0;                   /* print a usage message */
  }                             /* and abort the program */

  /* --- evaluate arguments --- */
  for (i = 1; i < argc; i++) {  /* traverse the arguments */
    s = argv[i];                /* get the next argument */
    if ((*s == '-') && *++s) {  /* if the argument is an option */
      while (*s) {              /* traverse the characters */
        switch (*s++) {         /* evaluate the option */
          case 'a': report |= ALL|ONE;               break;
          case 'r': report |= STEPS;                 break;
          case 'o': report |= ONE;                   break;
          case 'b': maxdep  = (int)strtol(s, &s, 0); break;
          case 'h': example();                              return  0;
          default : printf("unknown option '-%c'\n", *--s); return -1;
        }                       /* set the reporting mode */
      } }                       /* from the given options */
    else if (!fname)            /* if no file name is recorded yet, */
      fname = argv[i];          /* note the file name argument */
    else { printf("two file names given\n"); return -1; }
  }                             /* check for a given file name */
  if (!fname) { printf("no file name given\n"); return -1; }
  in = (*fname && (*fname != '-')) ? fopen(fname, "r") : stdin;
  if (in == NULL) {             /* open the input file */
    printf("cannot open '%s'\n", fname); return -1; }

  /* --- initialize the board --- */
  field = board +81;            /* traverse the Sudoku board */
  for (r = 9; --r >= 0; ) {     /* traverse the rows */
    sr = r -(r % 3);            /* compute the square row */
    for (c = 9; --c >= 0; ) {   /* traverse the columns */
      --field;                  /* set every field to unknown */
      field->digit = (short)(n = 0); field->row = (short)r;
      field->flags = 0x1ff;          field->col = (short)c;
      for (i = 0; i < 9; i++) { /* collect dependent fields */
        if (i != r) field->deps[n++] = board +i*9 +c;
        if (i != c) field->deps[n++] = board +r*9 +i;
      }                         /* (row and column) */
      sc = c -(c % 3);          /* compute the square column */
      for (i = sr +3; --i >= sr; )
        for (k = sc +3; --k >= sc; )
          if ((i != r) && (k != c)) field->deps[n++] = board +i*9 +k;
    }                           /* traverse the corresponding square */
  }                             /* and collect the dependent fields */

  /* --- read the puzzle --- */
  for (i = 0; i < 81; i++) {    /* traverse the fields */
    do {                        /* field read loop */
      k = fgetc(in);            /* read the next character */
      if (k < 0) { printf("read error on '%s'\n", fname); return -1; }
    } while (strchr(digits, k) == NULL);
    if ((k != '.') && (k != '0')) set(board +i, k-'0', GIVEN);
  }                             /* set the given numbers */
  if (in != stdin) fclose(in);  /* close the input file */
  show("puzzle", 0, -1);        /* print the Sudoku puzzle */

  /* --- check the puzzle --- */
  for (i = 81; --i >= 0; ) {    /* traverse the fields of the board */
    if (board[i].flags != 0) continue;
    if (board[i].digit != 0) printf("\nduplicate number in");
    else                     printf("\nno possible number for");
    printf(" row %d, column %d\n", board[i].row+1, board[i].col+1);
    return -1;                  /* if no number bit is set, */
  }                             /* there is no possible number */

  /* --- initialize the lookup table --- */
  for (i = 512; --i >= 0; )     /* traverse the lookup table */
    unique[i] = -1;             /* and clear its entries, then */
  unique[0x000] = 0;            /* store numbers for unique patterns */
  unique[0x001] = 1; unique[0x002] = 2; unique[0x004] = 3;
  unique[0x008] = 4; unique[0x010] = 5; unique[0x020] = 6;
  unique[0x040] = 7; unique[0x080] = 8; unique[0x100] = 9;

  /* --- initialize the board sections --- */
  for (r = 9; --r >= 0; ) {     /* traverse the rows */
    field = board +r*9;         /* get the board row */
    for (c = 9; --c >= 0; ) {   /* traverse the columns */
      sects[r  ][c] = field +c;
      sects[r+9][c] = board +c*9 +r;
    }                           /* collect the fields */
  }                             /* in the same row or column */
  for (n = 18, r = 0; r < 9; r += 3)
    for (c = 0; c < 9; n++, c += 3)
      for (i = 3; --i >= 0; )   /* collect the fields */
        for (k = 3; --k >= 0; ) /* in the same square */
          sects[n][i*3+k] = board +(r+i)*9 +c+k;

  /* --- solve the puzzle --- */
  if (maxdep >= 0) n = maxdep--;
  else { maxdep = 0; n = 81; }  /* get the range of maximum depths */
  while (++maxdep <= n) {       /* iterative deepening backtracking */
    k = solve(0);               /* try to solve the puzzle */
    if (k ==  0) return 0;      /* if a solution was found, abort */
    if (k == -1) { printf("\npuzzle is unsolvable\n"); return -1; }
    restore(0);                 /* check for an unsolvable puzzle, */
  }                             /* then restore the board state */
  printf("\npuzzle is unsolvable (with this backtracking depth)\n");
  return -1;                    /* return 'puzzle is unsolvable' */
}  /* main() */
